%% ==============
\chapter{Background}
\label{ch:Background}
%% ==============

%% ===========================
\section{Haskell}
\label{ch:Background:sec:section1}
%% ===========================

In this section the aspects of Haskell that are relevant for this thesis will
be outlined. For a more thorough introduction to Haskell consult for example
Hutton's ``Programming in Haskell''\cite{hutton07} or Davie's ``introduction to
functional programming systems using Haskell''\cite{davie92}.

Haskell is a \emph{functional} programming language, which means the basic
method of computation is the application of functions to arguments. In contrast
to this the basic method of computation of imperative programming languages is
changing stored values.\cite{hutton07}

To fully utilize functional programming functions are \emph{first class}
entities in Haskell, which means they

\begin{itemize}
\item can be named,
\item can be the value of some expression,
\item can be members of a list,
\item can be elements of a tuple,
\item can be passed to a function as a parameter and
\item can be returned from a function as its result.\cite{davie92}
\end{itemize}

Consider the following function definiton, taken from Haskell's base library,
as a short example of Haskell code, highlighting some of the language features
and syntax:
\begin{code}
-- || 'takeWhile', applied to a predicate @p@ and a list @xs@, returns the
-- longest prefix (possibly empty) of @xs@ of elements that satisfy @p@:
--
-- > takeWhile (< 3) [1,2,3,4,1,2,3,4] == [1,2]
-- > takeWhile (< 9) [1,2,3] == [1,2,3]
-- > takeWhile (< 0) [1,2,3] == []
--
\end{code}
These lines are comments, describing how the function works. Basic comments
start with the characters |--| and reach until the end of the line.
\savecolumns
\begin{code}
takeWhile                ::  (alpha -> Bool) -> [alpha] -> [alpha]
\end{code}
This line contains the function's type signature, which tells us that the
function we are defining takes another function of type |alpha -> Bool| as its
first argument, a list of type |[alpha]| as it's second argument and returns a
list of type |[alpha]|. \emph{takeWhile} is a \emph{polymorphic} function, as
|alpha| is a type variable that can be substituted with any concrete type.
\restorecolumns
\begin{code}
takeWhile _ []           =   []
\end{code}
Here we define the function \emph{takeWhile} for the case where the second
argument, the list to be passed, is empty. This is done using \emph{pattern
matching}: The second argument to \emph{takeWhile} has to fit the pattern
|[]|, which denotes the empty list.
\restorecolumns
\begin{code}
takeWhile p (x:xs)
            | p x        =   x : takeWhile p xs
            | otherwise  =   []
\end{code}
This is the second case of the \emph{takeWhile} function. Here the second
argument is pattern matched to be of the type \emph{(x:xs)}, which means that
\emph{x} is the first member of the list and \emph{xs} is the rest of the
list. The first argument, \emph{p}, is a function of type |alpha -> Bool|, which
means a function that takes a value of type |alpha| and returns a Boolean. \emph{p}
is called with the first list member \emph{x} as its argument. If the result
is |true|, a new list |x : takeWhile p xs| is returned. Otherwise an empty list
is returned.

Defining a function that maps a function to the values of a list is very easy
in Haskell:
\begin{code}
map             ::  (alpha -> beta) -> [alpha] -> [beta]
map  _  []      =   []
map  f  (x:xs)  =   f x : map f xs
\end{code}
An expression is said to have \emph{side effects} when it interacts with the
outside world, for example by reading from a file or modifying a global
variable. Haskell is a \emph{pure} programming language, which means side
effects are prohibited.\footnote{There is of course a way to cause side effects
in Haskell, the IO Monad.} The result of an expression depends solely on its
inputs, which means the evaluation of a pure function returns the same result
every time it is called with the same inputs, which leads us to the following
property:
\begin{quote}
  ``[\ldots] [I]f we wish to find the value of an expression which contains a
  sub-expression, the only thing we need to know about the sub-expression is
  its value. Any other features of the sub-expression, such as its internal
  structure, the number and nature of its components, the order in which they
  are evaluated or the colour of the ink in which they are written, are
  irrelevant to the value of the main expression.''\cite[page~19]{fundamental}
\end{quote}
This property is called \emph{referential transparency}\footnote{The term
\emph{referential transparency} has its origin in philosophy, where it means
that replacing a part of a sentence with another term that refers to the same
thing does not change the meaning of the sentence.\cite{quine}}, and as a
consequence of it expressions do not have to get evaluated in any specific
order. Another favourable effect of referential transparency is that it allows
clear equational reasoning.\cite{hudak89}

%%% ===========================
%\section{Learning Haskell}
%\label{ch:Background:sec:section2}
%%% ===========================
%
%hmm? Do we need this?

%% ===========================
\section{Lazy Evaluation}
\label{ch:Background:sec:section3}
%% ===========================

Haskell uses referential transparency by deferring evaluations to the latest
possible moment: When they are actually needed. This is called \emph{lazy
evaluation} and is the default evaluation strategy in Haskell.

Lazy evaluation automatically avoids unnecessary computations, because they
will never be needed. Consider this example:
\begin{code}
squares       ::  [Integer]
squares       =   map (^2) [1..10]

smallSquares  ::  [Integer]
smallSquares  =   takeWhile (<= 20) squares
\end{code}
The variable \emph{squares} is defined to be the squares of the numbers 1 to 10,
namely |[1, 4, 9, 16, 25, 36, 49, 64, 81, 100]|. \emph{smallSquares} is defined to
be the list of |squares| until the point where they grow greater than 20, namely
|[1, 4, 9, 16]|.

Let us think about what has to get evaluated to calculate the value of
|smallSquares|. As |smallSquares| stops when it encounters the first square number
that is greater than 20 only the first five list members of |squares| get
evaluated. The fifth member of |squares| gets evaluated to 25, which is checked
to be greater than 20 and therefore no further values are evaluated.

Lazy evaluation allows carrying this to an extreme, by defining infinite data
structures. Just as before they get only evaluated as far as they are needed.
Consider the following example:
\begin{code}
squares       =  map (^2) [1..]

smallSquares  =  takeWhile (<= 20) squares
\end{code}
The only difference to the example above is that \emph{squares} refers to the
list of all |squares| now. Just as before only the first five list members of
|squares| actually get evaluated.

Lazy evaluation is important for creating modular Haskell programs. A common
approach is to split a function into two modules: A generator that constructs
many possible solutions and a selector that checks whether those values
actually satisfy the conditions for being a solution.\cite{fpmatters}

%% ===========================
\section{Sharing}
\label{ch:Background:sec:section4}
%% ===========================

A consequence of Haskell being a pure programming language is that data is
\emph{immutable}. Values cannot be directly altered. Instead a copy containing
the modified data gets created. Because data never gets altered there is no
need to copy unmodified values. Instead they get \emph{shared} between data
structures. The above code illustrates this. As the \emph{smallSquares} contain
the first four values of \emph{squares} they are not copied, but instead the
old values are also used in |smallSquares|.

Sharing is a vital part of lazy evaluation. Because functions are first class
entities in Haskell they can also be shared.
\begin{code}
doubleTotal = total + total
  where total = sum [1..100]
\end{code}
Instead of evaluating the value for \emph{total} every time it is needed it is
only calculated once and when it is requested a second time the old value is
returned directly. Thanks to referential transparency sharing does not
influence the result of a computation.

Sharing is useful when designing performant functional algorithms, but is
rarely used consciously as sharing is unobservable in pure Haskell, which makes
it hard to understand and preserve.\footnote{Actually the Haskell 2010 Language
Report\cite{haskell2010} does not mention sharing at all, but it is very
practical. Because of referential transparency results will be the same whether
sharing is utilized or not.}\cite{trouble}

%% ===========================
\section{Glasgow Haskell Compiler}
\label{ch:Background:sec:section2}
%% ===========================

The Glasgow Haskell Compiler (\emph{GHC})\cite{ghc} is a compiler and
interactive environment for Haskell, which is mainly developed by Simon Peyton
Jones and Simon Marlow. GHC is available for several platforms and processor
architectures, has extensive optimizations and is probably the most popular
Haskell compiler. It is also the foundation of the \emph{Haskell Platform},
which considers itself ``the easiest way to get started with programming
Haskell''\cite{haskellplatform}. This makes it the optimal target for
ghc\hbox{-}vis, which is aimed at Haskell professionals and novices alike.

GHC's interactive environment, \emph{GHCi} provides a command line interface,
which gives immediate feedback for Haskell expressions entered. It can be used
to interactively evaluate Haskell expressions and interpret programs.

%% ===========================
\section{GHC Heap}
\label{ch:Background:sec:section5}
%% ===========================

In GHC all dynamic objects reside in the heap in the form of a
\emph{closure}.\footnote{A commonly used definition of a \emph{closure} is a
block of code together with its free variables. This is \emph{not} the
definition used in this thesis. Our definition, which is taken from
\cite{jones92}, is more general and includes all dynamic objects.} Closures are
laid out as shown in Figure~\ref{fig:closure-layout}. Static objects, which
already exist when the program is started, reside in the data section of
compiled object code.

\begin{figure} [ht]
  \centering
\begin{tikzpicture}
  \begin{scope} [start chain=going right,node distance=-0.15mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=3ex,text depth=1.2ex,
      inner sep=2ex,align=center
    },
  }
  \node (pointer) [on chain] {Pointer to closure};
  \node (header) [node distance=8ex, on chain, simple node] {Header};
  \node (payload) [on chain, simple node,text width=20ex] {Payload};

  \end{scope}
  \begin{scope}[start chain=going below, node distance=-0.15mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=3ex,text depth=1.2ex, text width=15ex,
      inner sep=2ex,align=center
    },
  }
  \node[node distance=4ex,on chain,below right=of header] {Info table};
  \node[node distance=1ex,on chain,simple node] {Layout};
  \node[on chain,simple node] {Closure Type};
  \node[on chain,simple node] {SRT Bitmap};
  \node (ec) [on chain,simple node,dashed] {Entry Code};
  \end{scope}

  \draw[-triangle 45] (pointer.east) -- (header.west);
  \draw[*-triangle 45] ($ (header.base) !.25! (header.south) $) ||- (ec.north west);

  \end{tikzpicture}
  \caption{General layout of a closure}
  \label{fig:closure-layout}
\end{figure}

%We are not interested in profiling information, which may additionally be
%stored in closures, so they will not be considered.

The header of a closure points to an info table which specifies
\begin{itemize}
  \item the \emph{layout} of the payload: How many words does it contain, how
        many of those are pointers,
  \item the \emph{type} of this particular closure, for example a data
        constructor, a function or a thunk,
  \item the \emph{SRT Bitmap}, a field used for garbage collection, which is of
        no further concern to us.
\end{itemize}

The relevant types of closures for this work are:\cite{ghc-heap}
\begin{description}
  \item[Data Constructors] provide basic data storage. This is for example
  used for Integers, Chars, the list constructor \emph{cons} (:), the boolean
  values \emph{True} and \emph{False} or user defined constructors like
  \emph{Leaf} and \emph{Node} in the following example:
  \begin{code}
data Tree  =  Leaf
           |  Node Tree Int Tree

t  ::  Tree
t  =   Node (Node Leaf 2 Leaf) 1 Leaf
  \end{code}
  The layout of \emph{t's} closure on the heap can be seen in
  Figure~\ref{fig:t-layout}.

\begin{figure} [ht]
  \centering
\begin{tikzpicture}
  \begin{scope} [start chain=going right,node distance=-0.15mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=3ex,text depth=1.2ex,
      inner sep=2ex,align=center
    },
  }
  \node (header) [node distance=8ex, on chain, simple node] {Header};
  \node (pointer1) [on chain, simple node] {Pointer};
  \node (pointer2) [on chain, simple node] {Pointer};
  \node (pointer3) [on chain, simple node] {Pointer};

  \end{scope}
  \begin{scope}[start chain=going below, node distance=-0.15mm]
  \tikzset{
    simple node/.style={
      draw,
      text height=2.6ex,text depth=1.2ex, text width=15ex,
      inner sep=2ex,align=center
    },
  }
  \node[node distance=4ex,on chain,below right=of header] {Info table};
  \node (condesc) [node distance=1ex,on chain,simple node,text height=,text depth=] {main::Interac-\\tive.Node};
  \node (layout) [on chain,simple node,text height=,text depth=] {3 pointers\\0 nonpointers};
  \node (type) [on chain,simple node] {CONSTR};
  \node (srt) [on chain,simple node] {1};
  \node (code) [on chain,simple node] {\texttt{return}};
  \end{scope}

  \node [node distance=1ex,right=of condesc] {Constructor Description};
  \node [node distance=1ex,right=of layout] {Layout};
  \node [node distance=1ex,right=of type] {Closure Type};
  \node [node distance=1ex,right=of srt] {Constructor Tag};
  \node [node distance=1ex,right=of code] {Code};

  \node (pointer2d) [node distance=4ex,above =of pointer2] {Int Closure};
  \node (pointer1d) [node distance=2ex,left=of pointer2d] {Node Closure};
  \node (pointer3d) [node distance=2ex,right=of pointer2d] {Leaf Closure};

  \draw[*-triangle 45] ($ (header.base) !.25! (header.south) $) ||- (srt.south west);
  \draw[*-triangle 45] ($ (pointer1.base) !.5! (pointer1.north) $) -- (pointer1d.south);
  \draw[*-triangle 45] ($ (pointer2.base) !.5! (pointer2.north) $) -- (pointer2d.south);
  \draw[*-triangle 45] ($ (pointer3.base) !.5! (pointer3.north) $) -- (pointer3d.south);

  \end{tikzpicture}
  \caption{Layout of Tree t's closure on the heap}
  \label{fig:t-layout}
\end{figure}

  The closure of \emph{t} contains a header, which tells us that there are 3
  pointers and 0 nonpointers in the closure, that we are dealing with a closure
  of Type CONSTR and that the constructor is called
  \emph{main::Interactive.Node}.
  \item[Function Closures] represent Haskell functions, without any arguments,
  but containing pointers to the function's free variables.
  % free variables?
  \item[Partial Applications] represent functions with some arguments applied
  to them, but less than they take in total.
  \item[Thunks] represent suspended computations, which can be evaluated
  further.

  When a thunk gets evaluated it is replaced with the result of its evaluation.
  \item[Generic Applications] are like partial applications, but all arguments
  are given. Therefore they are \emph{thunks} and can be evaluated. Generic
  Applications are mainly used in interpreted code.
  \item[Selector Thunks] perform simple selections. They are necessary in order
  to prevent space leaks, that would happen when a part of a data structure is
  selected, but another part of it is not needed anymore.\cite{wadler87}
  \item[Indirections] simply point to other closures. They are introduced when
  a thunk gets evaluated to point to its value and are necessary because it
  might not be possible to simply replace a thunk with its result when the
  resulting closure is too big.
  \item[Byte Code Objects] (BCOs) contain byte code that can be executed by the
  interpreter.
  \item[Blackhole Closures] appear when an evaluation is happening and later
  get replaced by the result. They are used to avoid space leaks that would
  happen when the closure which is getting evaluated points to closures that
  are not actually needed anymore. Blackhole closures prevent the garbage
  collector from discovering other pointers in the closure, which can then be
  made available for recycling.\cite{tailrecursion}

  % Need more?
\end{description}

%When we consider the form of an expression in the lambda calculus, ... head normal form ...

The library ghc-heap-view\cite{ghc-heap-view} by Joachim Breitner makes it
possible to investigate closures on the GHC heap directly in Haskell.

Closures may be moved around or evaluated at any moment by the garbage
collector, which only uses the information available on the
heap. GHC uses a two-space stop-and-copy collector, that divides the heap
memory into two spaces. To perform a garbage collection all currently used
closures are copied from one space to the the other. To realize which closures
to copy the garbage collector follows the pointers of closures.\cite{jones92}

When a closure is evaluated it gets updated with its \emph{weak-head normal
form} (WHNF), which means the outermost part of the expression is evaluated.
Note that this does not imply that the expression is in \emph{normal form}, for
which the expression would have to be fully evaluated. Data constructors,
function closures and partial applications are always in WHNF, because when
they are the top level closure they cannot be further evaluated. Thunks,
selector thunks and generic applications are not necessarily in WHNF and
therefore it is only those that can get evaluated.

There are two typical evaluation models that could be used in GHC:

\begin{description}
  \item[Push/Enter]: The caller pushes the function arguments on the stack and
  calls the function. The function itself figures out how many arguments it has
  been passed.
  \item[Eval/Apply]: The caller examines the function closure and calls the
  function with the right number of arguments. Three cases may occur:
  \begin{enumerate}
    \item If there are not enough arguments the function gets turned into a
    partial application.
    \item If the right number of arguments are given the function is simply
    called.
    \item If there are too many arguments, the function is applied to the
    arguments it can take and then the resulting function is examined in the
    same way.\cite{marlow04}
  \end{enumerate}
\end{description}

GHC uses the Eval/Apply evaluation model. We will look at GHC's behavior
concerning closures on the heap later, using the developed tool, ghc-vis.

%% ===========================
\section{Related Work}
\label{ch:Background:sec:section6}
%% ===========================

There have been some projects with regards to visualizing data structures
in Haskell. Some are to be used in a similar way as ghc-vis, in a live
environment, whereas others allow debugging code after it has been run.

%% ===========================
\subsection{GHCi Debugger}
\label{ss:debugger}
%% ===========================

%In the rest of this thesis many examples in GHCi will follow.

GHCi comes with its own debugger, which has been developed in 2006 under the
Google Summer of Code intiative by José Iborra.\cite{DSIC-II/09/07} The debugger provides the
\emph{:print} command. It inspects data structures at runtime without forcing
evaluation of them.
\begin{Verbatim}
$ ghci
GHCi, version 7.4.2: http://www.haskell.org/ghc/  :? for help
ghci> let a = [1..3]
ghci> :print a
a = (_t1::[Integer])
\end{Verbatim}
We can see that \emph{a} currently is completely unevaluated, it simply consists of a thunk.
\begin{Verbatim}
ghci> head a
1
ghci> :print a
a = 1 : (_t2::[Integer])
\end{Verbatim}
Now that we have peeked inside of \emph{a} it has been partially evaluated.
\begin{Verbatim}
ghci> head $ tail a
2
ghci> :print a
a = 1 : 2 : (_t3::[Integer])
ghci> a
[1,2,3]
ghci> :print a
a = [1,2,3]
\end{Verbatim}
We see that \emph{:print} enables us to see lazy evaluation in action.
Unfortunately sharing cannot be seen as easily:
\begin{Verbatim}
ghci> let b = a ++ a
ghci> head b
1
ghci> :print b
b = 1 : (_t3::[Integer])
ghci> b
[1,2,3,1,2,3]
ghci> :print b
b = [1,2,3,1,2,3]
\end{Verbatim}
Even though the numbers in \emph{b} are shared with \emph{a} and even within
\emph{b} itself \emph{:print} does not tell us about this in any way.

%% ===========================
\subsection{Vacuum}
%% ===========================

Another tool that has been developed to inspect data structures in Haskell is
vacuum.\cite{vacuum} It can tell us how values are shared within b:
\begin{Verbatim}
ghci> System.Vacuum.Cairo.view b
\end{Verbatim}
\begin{figure} [ht]
  \centering
  \includegraphics[height=.5\textheight]{images/vacuum1.png}
  \caption{Visualization of a list using vacuum}
  \label{fig:vacuum1}
\end{figure}
This causes the output seen in Figure~\ref{fig:vacuum1} to appear on the
screen. From this graph it is clear how the members of the list are shared
instead of being copied. Unfortunately vacuum-cairo evaluates data structures
fully before showing them. To also inspect lazy evaluation using vacuum we can
do the following:
\begin{Verbatim}
ghci> let a = "foo"
ghci> let b = a ++ a
ghci> head b
'f'
ghci> GHC.Vacuum.GraphViz.graphToDotFile "vacuum2"
         Data.GraphViz.Commands.Png $ GHC.Vacuum.nameGraph
         (GHC.Vacuum.vacuumLazy (a,b))
\end{Verbatim}
\begin{figure} [ht]
  \centering
  \includegraphics[width=.5\textwidth]{images/vacuum2.pdf}
  \caption{Lazy visualization using vacuum}
  \label{fig:vacuum2}
\end{figure}
Which leads to the output in Figure~\ref{fig:vacuum2}. We can see that the
value \emph{C\#l3} is referenced both in \emph{a} and \emph{b}. That's the
|'f'| which was evaluated by calling |head b|. What can not be seen is that the
\emph{l5} thunk references \emph{a} and has another value shared with \emph{a}.

Later, in Subsection~\ref{ss:Comparison}, we will come back to these examples
and compare ghc-vis with vacuum and :print to illustrate how ghc-vis solves the
issues that were just shown.

%% ===========================
\subsection{Hood / GHood}
%% ===========================

\emph{Hood}\cite{hood} observes Haskell data structures in GHC compiled
programs. \emph{GHood}\cite{ghood} creates an animation of the evaluation of
data structures which can be watched in a Java program after execution. It is
possible to step through the evaluation and observe how the data structure
changes, including the evaluation of internal thunks. See
Figure~\ref{fig:ghood} for an example.

\begin{figure} [ht]
  \centering
  \includegraphics[width=\textwidth]{images/ghood.png}
  \caption{Visualization of data structures using GHood}
  \label{fig:ghood}
\end{figure}

Sharing can not be observed in GHood and as an animation first has to be
created before one can visualize it, GHood can not be used in GHCi to visualize
live data structures.

%% ==============
\chapter{ghc-vis}
\label{ch:ghc-vis}
%% ==============

The main part of this thesis was the design and implementation of
\emph{ghc-vis}, a tool to visualize lazy evaluation and sharing in Haskell. In
this chapter the design, implementation and challenges of \emph{ghc-vis} will
be discussed.

%% ===========================
\section{Design}
\label{ch:Vis:sec:section1}
%% ===========================

In order to be useful ghc-vis has to provide two functions:

\begin{itemize}
  \item Variables defined in GHCi can be supplied to ghc-vis for visualization.
  \item When a closure is clicked in ghc-vis' visualization window GHCi
  evaluates it.
\end{itemize}

There are multiple possibilities for the high level integration of ghc-vis
into GHCi, which could achieve those functions:

\begin{enumerate}
  \item Modify the source code of GHCi. This allows for easy bidirectional
  communication between ghc-vis and GHCi.
  \item Supply commands to GHCi on stdin.
  \item Run a thread inside of GHCi.
\end{enumerate}

Because of its simplicity the third approach was chosen. ghc-vis is run
directly inside of GHCi in a seperate thread. The graphical interface is
created using the Haskell library bindings to GTK, Cairo and
Pango.\cite{gtk2hs}

The interface to ghc-vis from GHCi is defined by GHCi commands that map to
simple Haskell functions. The main commands are listed and explained in
Table~\ref{tab:commands}.

\begin{table} [ht]
  \centering
  \begin{tabular}{lp{11.5cm}}
    \toprule
    Command & Description \\
    \midrule
    \texttt{:vis} & Run the visualization window \\
    \texttt{:view x} & Add the Haskell expression \emph{x} to the visualization window \\
    \texttt{:eval t} & Force evaluation to WHNF of the thunk named \emph{t} \\
    \texttt{:switch} & Switch between the visualization type (linear, graph) \\
    \texttt{:update} & Update the visualization window, in case an expression has changed \\
    \texttt{:clear} & Remove all visualized expressions \\
    \texttt{:export file} & Export the current visualization to a file;
    \emph{SVG}, \emph{PDF}, \emph{PS} and \emph{PNG} supported \\
    \bottomrule
  \end{tabular}
  \caption{Main commands of ghc-vis}
  \label{tab:commands}
\end{table}

%Further functions are:
%
%:su

%To supply a variable to be visualized it is simply 

We will now take a quick look at the usage of ghc-vis to illustrate how it
works, before we continue discussing it.

\subsection{Demonstration}
\label{ss:Demonstration}

We begin by running ghci and starting ghc-vis inside of it. User input is
prepended by \texttt{ghci>}. All other lines are output by GHCi:
\begin{Verbatim}
$ ghci
GHCi, version 7.4.2: http://www.haskell.org/ghc/  :? for help
ghci> :vis
[Package loading output cut]
\end{Verbatim}
The visualization window, which currently does not show anything. We
can add expressions to the visualization:
\begin{Verbatim}
ghci> let a = [1..3]
ghci> :view a
ghci> let b = cycle a
ghci> :view b
ghci> :view "foo" ++ "bar"
\end{Verbatim}
\begin{figure} [ht]
  \centering
  \includegraphics[scale=.5]{images/1.pdf}
  \caption{Demonstration of ghc-vis}
  \label{fig:1}
\end{figure}
%\begin{figure} [ht]
%  \begin{minipage}{0.45\textwidth}
%    \begin{Verbatim}
%ghci> let a = [1..3]
%ghci> :view a
%ghci> let b = cycle a
%ghci> :view b
%ghci> :view "foo" ++ "bar"
%    \end{Verbatim}
%  \end{minipage}
%  \begin{minipage}{0.55\textwidth}
%    \includegraphics[scale=.5]{images/1.pdf}
%  \end{minipage}
%  \caption{Demonstration of ghc-vis}
%  \label{fig:1}
%\end{figure}
\begin{table} [ht]
  \centering
  \begin{tabular}{lcl}
    \toprule
    Name & Object & Meaning\\
    \midrule
    Unnamed & \begin{minipage}{0.1\textwidth}\centering\includegraphics[scale=0.5]{images/unnamed.pdf}\end{minipage} & Represents Data Constructors \\
    Thunk & \begin{minipage}{0.1\textwidth}\centering\includegraphics[scale=0.5]{images/thunk.pdf}\end{minipage} & Represents Thunks, Generic Applications \\
    Function & \begin{minipage}{0.1\textwidth}\centering\includegraphics[scale=0.5]{images/function.pdf}\end{minipage} & Represents Functions, Partial Applications \\
    Named & \begin{minipage}{0.1\textwidth}\centering\includegraphics[scale=0.5]{images/block.pdf}\end{minipage} & A named object, so that it can be linked elsewhere \\
    Link & \begin{minipage}{0.1\textwidth}\centering\includegraphics[scale=0.5]{images/link.pdf}\end{minipage} & Links to an object which is already visualized \\
    \bottomrule
  \end{tabular}
  \caption{Linear view objects}
  \label{tab:list-types}
\end{table}
The output in Figure~\ref{fig:1} appears in the visualization window. What
we're seeing now is the \emph{linear view}. It shows values in a linear form,
similar to \emph{:print}, but with graphical highlighting and deeper
inspections. See Table~\ref{tab:list-types} for the mapping of linear view
objects to actual heap closures.

In Figure~\ref{fig:1} we can see that \emph{a} is referenced by \emph{b},
because the thunk \emph{t0} is linked in \emph{b}. Hovering an object
highlights all other objects in the visualization window which link to this
object or are linked by it. In order to force evaluation of an object in the
visualization we can click on it or enter the following in GHCi:
\begin{Verbatim}
ghci> :eval t1
\end{Verbatim}
\begin{figure} [ht]
  \centering
  \includegraphics[scale=.8]{images/2.pdf}
  \caption{Demonstration of ghc-vis}
  \label{fig:2}
\end{figure}
We see in Figure~\ref{fig:2} that the first member of \emph{b} has been
evaluated.  It's called \emph{b0} and is a reference to the value that's also
referenced as the head of \emph{a}, as they share the same name. \emph{S\# 1}
stands for Integer data constructor for small Integers, which is called
\emph{S\#} in GHC and its argument \emph{1}.

The linear view may not always be ideal, especially for bigger data structures.
The heap can be thought of as a graph with closures as vertices and their
pointers to each other as directed edges. This is represented in the
\emph{graph view} of ghc-vis, which we can switch to like this:
\begin{Verbatim}
ghci> :switch
\end{Verbatim}
\begin{figure} [ht]
  \centering
  \includegraphics[scale=.4]{images/3.pdf}
  \caption{Demonstration of ghc-vis}
  \label{fig:3}
\end{figure}
The output in Figure~\ref{fig:3} appears on the screen.

When an object is updated by accessing it \emph{:update} has to be called to
refresh the visualization window. Alternatively any object can be clicked to
perform an update.
\begin{Verbatim}
ghci> a !! 2
3
ghci> :update
\end{Verbatim}
\begin{figure} [ht]
  \centering
  \includegraphics[scale=.4]{images/4.pdf}
  \caption{Demonstration of ghc-vis}
  \label{fig:4}
\end{figure}
This causes the visualization window to be updated to the view in
Figure~\ref{fig:4}.

\subsection{Implementation}

ghc-vis has been implemented to work with the Glasgow Haskell Compiler
version~7.4.2. More recent versions of the GHC will likely require adjustments
in ghc-vis because of the interaction with GHC internals which are likely to
change in the future.

In order to analyze the representation of data structures on the heap we
represent closures on the heap as a \emph{HeapEntry} tuple and the part of the
heap we are interested in as a \emph{HeapMap}:
\begin{code}
data Box        =  Box Any

type HeapEntry  =  (Maybe String, Closure)
type HeapMap    =  [(Box, HeapEntry)]
\end{code}
The first element of a \emph{HeapEntry} identifies the entry on the heap.
It is used to denote top-level closures and closures we have already visited in
order to prevent infinite recursions. The second element is the closure,
containing the parsed information of the raw heap closure we actually are
interested in.

The \emph{HeapMap} is a simple list, identifying raw Haskell expressions with
their corresponding closure information. We want to use the HeapMap to look up
the corresponding \emph{HeapEntries} of \emph{Box}es, which is only possible in
$O(n)$ in a list of length $n$:
\begin{code}
lookup                 ::  Eq a => a -> [(a,b)] -> Maybe b
lookup  _    []        =   Nothing
lookup  key  ((x,y):xys)
    | key == x         =   Just y
    | otherwise        =   lookup key xys
\end{code}
Let us investigate why we are using a \emph{List} and equality (type class
\emph{Eq}) based comparisons instead of a totally ordered (type class
\emph{Ord}) \emph{Map}, which performs \emph{lookup}s in $O(log n)$ on a
balanced binary tree:
\begin{code}
data Map k a  =  Tip
              |  Bin {-# UNPACK #-} !Size !k a !(Map k a) !(Map k a)

lookup      ::  Ord k => k -> Map k a -> Maybe a
lookup k t  =   case t of
      Tip  ->  Nothing
      Bin _ kx x l r
           ->  case compare k kx of
                  LT  ->  lookup k l
                  GT  ->  lookup k r
                  EQ  ->  Just x
\end{code}
As the type signature of \emph{Map}'s lookup tells us, we need to provide an
instance of \emph{Ord} for the \emph{Box} data type, thereby defining a total
order of \emph{Box}es.
\begin{Verbatim}
ghci> let x = asBox 3 :: Box
ghci> let y = asBox 4 :: Box
ghci> x
0x00007f8ee7f796b8
ghci> y
0x00007f8ee7f5a6b8
ghci> System.Mem.performGC
ghci> x
0x00007f8ed0ffb090
ghci> y
0x00007f8ed105eaa0
\end{Verbatim}
First we defined two new \emph{Box}es. Then we print their actual memory
locations. After we perform a garbage collection (\emph{performGC}) we print
their memory locations again. The garbage collector run switched the values'
order in the memory around. As we have seen in
Section~\ref{ch:Background:sec:section5} the garbage collector is allowed to do
that at any time. Therefore comparing the values' memory locations does not
help us in establishing any consistent order between them. There is no other
information about the stored value available except its memory location.

StableName\cite{stablename} is a data type that is supposed to provide stable
ordering of pointers according to the paper in which they are introduced, but
unfortunately the current implementation of them does not instantiate the
\emph{Ord} type class. If the naive list implementation turned out as a
performance bottle neck a hash table with StableNames could be used instead of
it.

Creating the \emph{HeapMap} at first seems like the problem already solved by
\emph{reifyGraph}\cite{sharing}, a function to observe sharing in Haskell. It
works by instantiating the \emph{MuRef} type class for the data type one wants
to observe, thereby defining how to traverse the data structure. The vertex and
edge labels in the graph created are also user defined.

A closer look reveals that we have to deal with a number of problems that do
not exist when using \emph{reifyGraph}, which mostly stem from these
differences:

\begin{itemize}
  \item We want more than observing sharing, namely keep unevaluated data
  structures unevaluated to visualize lazy evaluation.
  \item We operate on a lower level, looking inside the heap.
  \item We do not want the user to supply any information about what he is
  going to visualize.
\end{itemize}

Some of the resulting problems are described in
Section~\ref{ch:ghc-vis:sec:section2}.

Our function for creating the \emph{HeapMap} from the actual heap, called
\emph{walkHeap}, works as follows:

\begin{enumerate}
  \item Add a dummy node to the HeapMap that links to all expressions to be
  visualized
  \item Add the expressions to the HeapMap
  \item Start a recursive heap walk along the pointers of the heap
  representations of the supplied expressions:
  \begin{enumerate}
    \item If we have already added this closure to the HeapMap, return
    \item Add the closure to the HeapMap
    \item Visit all relevant pointers the closure has
  \end{enumerate}
\end{enumerate}

%% ===========================
\section{Views}
\label{ch:Vis:sec:section3}
%% ===========================

%% ===========================
\subsection{Linear View}
%% ===========================

The \emph{linear view} is similar to \emph{:print}. In order to create it a
linear list of \emph{VisObject}s is generated from the HeapMap.
\begin{code}
data VisObject   =   Unnamed   String
                 |   Named     String [VisObject]
                 |   Link      String
                 |   Thunk     String
                 |   Function  String
          deriving Eq
\end{code}
\begin{table} [ht]
  \centering
  \begin{tabular}{lc}
    \toprule
    VisObject & Visualization \\
    \midrule
    \savecolumns
    \begin{code}Unnamed   "S# 1"\end{code} & \begin{minipage}{0.1\textwidth}\centering\includegraphics[scale=0.6]{images/unnamed.pdf}\end{minipage} \\
    \restorecolumns
    \begin{code}Named     "b0" [Unnamed "S# 1"]\end{code} & \begin{minipage}{0.1\textwidth}\centering\includegraphics[scale=0.6]{images/block.pdf}\end{minipage} \\
    \restorecolumns
    \begin{code}Link      "b0"\end{code} & \begin{minipage}{0.1\textwidth}\centering\includegraphics[scale=0.6]{images/link.pdf}\end{minipage} \\
    \restorecolumns
    \begin{code}Thunk     "t0"\end{code} & \begin{minipage}{0.1\textwidth}\centering\includegraphics[scale=0.6]{images/thunk.pdf}\end{minipage} \\
    \restorecolumns
    \begin{code}Function  "f0"\end{code} & \begin{minipage}{0.1\textwidth}\centering\includegraphics[scale=0.6]{images/function.pdf}\end{minipage} \\
    \bottomrule
  \end{tabular}
  \caption{VisObjects}
  \label{tab:list-types-2}
\end{table}
The VisObjects directly correspond to the objects drawn in the visualization,
as can be seen in Table~\ref{tab:list-types-2}.

The list of VisObjects gets created in this way:

\begin{enumerate}
  \item Create a HeapMap
  \item Recursively walk the HeapMap for every expression to be visualized:
  \begin{enumerate}
    \item If the current closure already has a name, return a link to the name
    \item Create a corresponding \emph{VisObject} for the closure
    \item Append the result of recursively walking the pointers of the closure
          to the VisObject we already have
    \item If the VisObject has multiple pointers pointing to it and is not
          already named, create a Named VisObject to surround the entire list
          of VisObjects
    \item Return the resulting list of VisObjects
  \end{enumerate}
\end{enumerate}

%% ===========================
\subsection{Graph View}
%% ===========================

The graph view is similar to \emph{vacuum-cairo}. If ghc-vis used a
non-interactive graph view the process of creating the graph would look like
this:

\begin{enumerate}
  \item Convert HeapMap to DotGraph
  \item Use graphviz\cite{graphviz} to convert DotGraph to SVG
  \item Draw resulting SVG
\end{enumerate}

But in order to support interactivity we had to take another course:

\begin{enumerate}
  \item Convert HeapMap to DotGraph
  \item Use graphviz to convert DotGraph to XDotGraph
  \item Parse drawing operations of XDotGraph
  \item Execute drawing operations, allowing for manipulation by user input
  (hovering, clicking)
\end{enumerate}

Steps 3 and 4 have been externalized into a seperate library called
\emph{xdot}. It uses the Haskell graphviz library
bindings\cite{graphviz-haskell} to convert a regular graph to a DotGraph,
supply it to the \texttt{dot} program and finally parse the resulting XDot file
in again.

As the Haskell graphviz library does not support parsing XDot
operations\cite{xdot}, this functionality was added to xdot. The relevant data
type used for this is \emph{Operation}, which translates all XDot operations
into a Haskell data type:
\begin{code}
data Operation =
   Ellipse   {  xy         :: Point
             ,  w          :: Double
             ,  h          :: Double
             ,  filled     :: Bool
             } |

   Polygon   {  points     :: [Point]
             ,  filled     :: Bool
             } |

   Polyline  {  points     :: [Point]
             } |

   BSpline   {  points     :: [Point]
             ,  filled     :: Bool
             } |

   Text      {  baseline   :: Point
             ,  alignment  :: Alignment
             ,  width      :: Double
             ,  text       :: String
             } |

   Color     {  rgba       :: (Double, Double, Double, Double)
             ,  filled     :: Bool
             } |

   Font      {  size       :: Double
             ,  name       :: String
             } |

   Style     {  style      :: String
             } |

   Image     {  xy         :: Point
             ,  w          :: Double
             ,  h          :: Double
             ,  name       :: String
             }

  deriving Show
\end{code}
All of the operations except for \emph{Image}, which integrates an external
image into the graph, but was not needed for ghc-vis, are supported for viewing
inside a GTK window using Cairo and Pango.\cite{gtk2hs}

%% ===========================
\section{Problems and Solutions}
\label{ch:ghc-vis:sec:section2}
%% ===========================

%Working close to the base of GHC can lead to some unexpected

%% ===========================
\subsection{Garbage Collector}
%% ===========================

When we have stored the layout of the heap into our representation of it, the
\emph{HeapMap}, the garbage collector may cause the \emph{HeapMap} to become
corrupted. Let us look at an example, for reasons of brevity some members of
the heap map are omitted:
\begin{Verbatim}
ghci> let x = [1..2]
ghci> hm <- walkHeap [(asBox x,"x")]
ghci> hm
[(0x00007f732dac52f0,(Nothing,BCOClosure {info = StgInfotable
  {ptrs = 4, nptrs = 0, tipe = BCO, srtlen = 0},
  instrs = 0x00007f732dac55a0, literals = 0x00007f732dac55e8,
  bcoptrs = 0x00007f732dac5618, arity = 0, size = 6, bitmap = 0}))
,(0x00007f732dac5380,(Just "x",APClosure {info = StgInfotable
  {ptrs = 0, nptrs = 0, tipe = AP, srtlen = 0}, arity = 116,
  n_args = 0, fun = 0x00007f732dac52f0, payload = []}))
,...]
ghci> x
[1,2]
ghci> hm2 <- walkHeap [(asBox x, "x")]
ghci> hm2
[...
,(0x00007f732e8dbc40/2,(Nothing,ConsClosure {info = StgInfotable
  {ptrs = 2, nptrs = 0, tipe = CONSTR_2_0, srtlen = 1},
  ptrArgs = [0x00007f732e8dbce0,0x00007f732e8dbcc8/2],
  dataArgs = [], pkg = "ghc-prim", modl = "GHC.Types", name = ":"}))
,(0x00007f732dac5380,(Just "x",BlackholeClosure {info = StgInfotable
  {ptrs = 1, nptrs = 0, tipe = BLACKHOLE, srtlen = 0},
  indirectee = 0x00007f732e8dbc40/2}))
,...]
\end{Verbatim}
We see that the APClosure at memory address \texttt{0x00007f732dac5380} was
replaced with a BlackholeClosure which points to the ConsClosure that resulted
from evaluating the generic application. After waiting for a few seconds
another look at \emph{hm2} reveals a surprising result:
\begin{Verbatim}
ghci> hm2
[...
,(0x00007f732dcad4b0/2,(Nothing,ConsClosure {info = StgInfotable
  {ptrs = 2, nptrs = 0, tipe = CONSTR_2_0, srtlen = 1},
  ptrArgs = [0x00007f732dcad890,0x00007f732dcad878/2],
  dataArgs = [], pkg = "ghc-prim", modl = "GHC.Types", name = ":"}))
,(0x00007f732dcad4b0/2,(Just "x",BlackholeClosure {info = StgInfotable
  {ptrs = 1, nptrs = 0, tipe = BLACKHOLE, srtlen = 0},
  indirectee = 0x00007f732dcad4b0/2}))
,...]
\end{Verbatim}
Suddenly the ConsClosure and the BlackholeClosure have the same memory
location: \texttt{0x00007f732dcad4b0/2}. How did this occur?

The BlackholeClosure was only created while the evaluation was happening to
prevent space leaks. When the garbage collector ran in the few seconds we
waited it replaced the BlackholeClosure, which has become useless now that the
evaluation has finished, with the resulting ConsClosure. That means the
BlackholeClosure isn't actually on the heap anymore, but our HeapMap wasn't
updated.

To deal with problems introduced by the garbage collector running in the time
between creating a HeapMap and visualizing it, a garbage collection is
performed right before creating a HeapMap.

%% ===========================
\subsection{Changing Pointers}
%% ===========================

When the garbage collector runs not after we have created a HeapMap, but
\emph{during} the creation, even bigger problems may appear. The same problems
can occur when a user evaluates data structures while the HeapMap gets created.

An example for this problem works as follows:

\begin{enumerate}
  \item We start parsing a closure from the heap into a Haskell data structure
  \item Closure gets evaluated by the garbage collector or a user
  \item The closure on the heap gets replaced by another closure
  \item We continue parsing the closure, now reading wrong values and pointers
  because we assume the closure to be of the old type
\end{enumerate}

In order to deal with problems of this kind failures in \emph{walkHeap} are not
fatal, but instead trigger a rerun.

%% ===========================
\subsection{Byte Code Objects}
%% ===========================

Byte Code Objects (BCOs) on the heap may link to rather complex data
structures, as in Figure~\ref{fig:bco:1}, which we are not interested in:
\begin{itemize}
  \item List of instructions to be executed
  \item List of pointers that are used in the instructions (often including
  type class information)
\end{itemize}
Therefore we cannot just show all information of the heap. Simply hiding them
doesn't work either because they may help in understanding sharing, as we can
see in Figure~\ref{fig:bco:2}, where |sum x| is referencing \emph{x} through
its BCO. Instead we traverse the BCO's pointers to see whether they point to an
object we would visualize anyway, and only connect it in that case, as can be
seen in Figure~\ref{fig:bco:3}.

\begin{figure} [ht]
  \begin{minipage}{0.78\textwidth}
  \begin{subfigure}{\textwidth}
    \centering
    \includegraphics[width=\textwidth]{images/bco.pdf}
    \caption{All BCO information}
    \label{fig:bco:1}
  \end{subfigure}
  \end{minipage}
  \begin{minipage}{0.2\textwidth}
  \begin{subfigure}{\textwidth}
    \centering
    \includegraphics[scale=.3]{images/bco2.pdf}
    \caption{No BCO information}
    \label{fig:bco:2}
  \end{subfigure}
  \begin{subfigure}{\textwidth}
    \centering
    \includegraphics[scale=.3]{images/bco3.pdf}
    \caption{Selective BCO information}
    \label{fig:bco:3}
  \end{subfigure}
  \end{minipage}
  \caption{Sharing through BCO pointers}
  \label{fig:bco}
\end{figure}

%% ===========================
\subsection{Loading and Reloading Files}
%% ===========================

When a file gets \texttt{:load}ed or \texttt{:reload}ed in GHCi all defined
variables get removed. It also causes the global communication channel ghc-vis
uses to be reinitialized. The thread listening to it would not receive any
further commands. Therefore the listening thread has a timeout that handles
this by resetting our global variables and listening to the new signal channel:
\begin{code}
react canvas window = do
  mbSignal <- timeout signalTimeout (takeMVar visSignal)
  case mbSignal of
    Nothing      -> do
      running <- readMVar visRunning
      if running
        then  react canvas window
        else do -- :r caused visRunning to be reset
              swapMVar visRunning True
              timeout signalTimeout (putMVar visSignal UpdateSignal)
              react canvas window
    Just signal  -> do
      -- Normal signal handling
\end{code}
