\chapter{Evaluation}
\label{ch:Evaluation}

\section{Comparison}
\label{ss:Comparison}

Let us return to the example we used in Section~\ref{ss:debugger} to illustrate the
shortcomings of \emph{:print} and \emph{vacuum} in visualizing lazy evaluation
and sharing and compare those results to \emph{ghc-vis}:

\begin{Verbatim}
ghci> let a = "foo"
ghci> let b = a ++ a
ghci> head b
'f'
\end{Verbatim}

\begin{figure} [ht]
  \centering
  \begin{subfigure}[b]{.49\linewidth}
    \begin{Verbatim}
ghci> :print a
a = 'f' : (_t1::[Char])
ghci> :print b
b = 'f' : (_t2::[Char])
    \end{Verbatim}
    \caption{:print}
  \end{subfigure}
  \begin{subfigure}[b]{.49\linewidth}
    \centering
    \includegraphics[scale=.4]{images/lists-linear.pdf}
    \caption{ghc-vis: Linear view}
  \end{subfigure}
  \caption{Comparing visualization of lists in \emph{:print} and the linear view of \emph{ghc-vis}}
  \label{fig:lists1}
\end{figure}

In Figure~\ref{fig:lists1} we see the output of \emph{:print} and the linear
view of \emph{ghc-vis}. The linear view contains the following information
which is missing from \emph{:print}:

\begin{itemize}
  \item The first character of \emph{a} and \emph{b}, |'f'|, is shared.
  \item The thunk for the remaining list in \emph{b} references \emph{a} and a
  function that is also refrenced in \emph{a}.
\end{itemize}

\begin{figure} [ht]
  \centering
  \includegraphics[scale=.4]{images/repeat.pdf}
  \caption{Visualizing |repeat 1| using ghc-vis}
  \label{fig:repeat}
\end{figure}

\emph{:print} can not be used to analyze cyclic data structures, like |repeat
1|. It keeps recursing on the data structure to build the textual
representation of it until it runs out of memory. They pose no problem for
ghc-vis, as can be seen in Figure~\ref{fig:repeat}, which shows how ghc-vis
visualizes |repeat 1|.

\begin{figure} [ht]
  \centering
  \subcaptionbox{vacuum}
    {\includegraphics[width=.5\textwidth]{images/vacuum2.pdf}}
  \subcaptionbox{ghc-vis: Graph view}
    {\includegraphics[scale=.4]{images/lists-graph.pdf}}
  \caption{Comparing visualization of lists in \emph{vacuum} and the graph view of \emph{ghc-vis}}
  \label{fig:lists2}
\end{figure}

In Figure~\ref{fig:lists2} we see the output of \emph{vacuum} and the graph
view of \emph{ghc-vis}. While the sharing of |'f'| is visible in \emph{vacuum}
as well, it is missing the referencing of \emph{a} and a function inside of
\emph{a} in \emph{b}'s thunk. The reason for this is that vacuum does not
inspect the insides of thunks. Vacuum does not support any interactivity, while
ghc-vis can evaluate thunks interactively and more data structures can easily
be added to its visualization.

\section{Visualizations}
\label{ch:Evaluation:sec:section1}

As the main focus of ghc-vis is visualizing Haskell data structures and
furthering the understanding of lazy evaluation and sharing we will now look at
examples using ghc-vis and evaluate what we can learn from them. No new results
will be presented in this section, instead we look at well known phenomenons.

\subsection{Compiled and Interpreted Code}

Compiled and interpreted code may lead to different representations on the
heap. Consider the following example:
\begin{code}
bigSquares  ::  [Integer]
bigSquares  =   filter (> 10) squares

squares     ::  [Integer]
squares     =   map (^2) [1..]
\end{code}
\begin{figure} [ht]
  \centering
  \subcaptionbox{Compiled code (-fobject-code)}
    {\includegraphics[scale=.35]{images/squares-compiled.pdf}}
  \subcaptionbox{Interpreted code (-fbyte-code)}
    {\includegraphics[scale=.35]{images/squares-interpreted.pdf}}
  \caption{Squares}\label{fig:squares}
\end{figure}
By default all code is interpreted in GHCi. Using \texttt{:set -fobject-code}
GHCi can be instructed to compile \texttt{:load}ed files instead. In
Figure~\ref{fig:squares} we see that the compiled version created special
functions, whereas the interpreted version uses more general functions, to
which some arguments were already supplied.

In the following examples the visualization is mostly the same for compiled and
interpreted code. A notable exception is the Sieve of Eratosthenes in
Subsection~\ref{ss:sieve} which is visualized in compiled form because the type
class information in the interpreted data structures causes the graph to be
significantly bigger.

Another notable difference between compiled and interpreted code is the
representation of data constructors with no arguments:
\begin{code}
  data Maybe a = Just a | Nothing
\end{code}
The data constructor for |Just| looks the same in compiled and interpreted
code. But data constructors without arguments, like |Nothing| in this example,
get a bogus argument in interpreted code, while in compiled code they have no
such argument.

The reason for this is that all closures on the heap have a minimum size of two
words, so that they can be replaced by pointers that temporarily get created
during garbage collection.\cite{ghc-heap} All interpreted data constructors are
allocated on the heap, and therefore those without arguments are filled with a
dummy argument to reach the minimum size.

Compiled data constructers are not allocated on the heap, but instead reside in
the data section of the compiled object code. They can not be replaced and
therefore they can be smaller than the minimum size for heap objects and no
bogus argument is added.

\subsection{Sharing}

This simple example shows how sharing may be used:
\begin{code}
complexOperation    ::  Double -> Double
complexOperation x  =   x + 1

f      ::    Double -> Int -> [Double]
f x 0  =     []
f x y  =     complexOperation x : f x (y-1)

g      ::    Int -> [Double]
g y    =     f 235 y

f'     ::    Double -> Int -> [Double]
f' x   =     go (complexOperation x) y
  where  go x' 0 = []
         go x' y = x' : go x' (y-1)

g'     ::    Int -> [Double]
g' y   =     f' 235 y
\end{code}
\begin{figure} [ht]
  \centering
  \subcaptionbox{Linear view}
    {\includegraphics[scale=.6]{images/sharing-linear.pdf}}
  \subcaptionbox{Graph view}
    {\includegraphics[scale=.4]{images/sharing-graph.pdf}}
  \caption{Sharing}\label{fig:sharing}
\end{figure}
Whereas \emph{g} does not use sharing \emph{g'} only calculates the
\emph{complexOperation} once and then shares the resulting value across the
entire list. This can be seen in Figure~\ref{fig:sharing}: \emph{g 3} consists
of three seperate list members, while \emph{g' 3} only uses a single one three
times.

\begin{table} [ht]
  \centering
  \begin{tabular}{lrr}
    \toprule
    Expression & Time spent & Memory used \\
    \midrule
    \savecolumns
    \begin{code}sum (g   1000000)\end{code} & 5.39 s &  1367.92 MiB \\
    \restorecolumns
    \begin{code}sum (g'  1000000)\end{code} & 1.55 s &   429.46 MiB \\
    \bottomrule
  \end{tabular}
  \caption{Effect of sharing on performance}\label{tab:sharing}
\end{table}

Table~\ref{tab:sharing} shows how time and memory performance are influenced in
this case by the use of sharing. Note that for the performance measurements the
following definition of \emph{complexOperation} was used instead:
\begin{code}
complexOperation x = x ** 9 - sqrt (x ** 5) + sqrt (sqrt (x ** 3))
\end{code}
A more complex operation increases the effect of sharing.

\subsection{Infinite Data Structures}
\label{ss:infinite}
We already looked at two definitions of the infinite list of ones in the
Introduction. Now we look at the following definition for the infinite list of
Fibonacci numbers:
%\begin{code}
%ones  :: [Integer]
%ones  = 1 : ones
%\end{code}
%The infinite list of ones defined as \emph{ones} simply references itself, as
%can be seen in Figure~\ref{fig:ones}.
%\begin{figure} [ht]
%  \centering
%  \subcaptionbox{Linear view}
%    {\includegraphics[scale=.2]{images/ones-linear.pdf}}
%  \subcaptionbox{Graph view}[3cm]
%    {\includegraphics[scale=.4]{images/ones-graph.pdf}}
%  \caption{Infinite list of ones}\label{fig:ones}
%\end{figure}
\begin{code}
fib  ::  [Integer]
fib  =   1 : 1 : [ a + b | (a,b) <- zip fib (tail fib) ]
\end{code}
\begin{figure} [ht]
  \centering
  \subcaptionbox{Linear view}
    {\includegraphics[scale=.6]{images/fib-linear.pdf}}
  \subcaptionbox{Graph view}
    {\includegraphics[scale=.4]{images/fib-graph.pdf}}
  \caption{Fibonacci numbers}\label{fig:fib}
\end{figure}
The state of \emph{fib} after evaluating the first three members can be seen
in Figure~\ref{fig:fib}. It becomes apparent how the function that constructs
member $n$ references members $n-1$ and $n-2$.

%\subsection{Fibonacci Trees}
%
%\begin{code}
%data Tree alpha beta  =  Leaf alpha
%                      |  Branch beta (Tree alpha beta) (Tree alpha beta)
%         deriving Show
%
%fibT    ::  Int -> Tree Integer Integer
%fibT n  =   fibsT !! (n-1)
%
%fibsT   ::  [Tree Integer Integer]
%fibsT   =   Leaf 0 : Leaf 0 : zipWith3 Branch [1..] fibsT (tail fibsT)
%\end{code}
%
%\begin{figure} [ht]
%  \centering
%  \subcaptionbox{Linear view}
%    {\includegraphics[scale=.5]{images/fibtree-linear.pdf}}
%  \subcaptionbox{Graph view}
%    {\includegraphics[scale=.4]{images/fibtree-graph.pdf}}
%  \caption{Fibonacci Trees}\label{fig:fibtree}
%\end{figure}
%
%This code to build Fibonacci Trees uses space $O(n)$ to store the first $n$
%trees by extensive sharing. Only one new element is added for each new tree, as
%can be seen in figure~\ref{fig:fibtree}.

\subsection{Sieve of Eratosthenes}
\label{ss:sieve}

The sieve of Eratosthenes is a simple algorithm to calculate all prime numbers.
It is often used as an example of lazy evaluation.
\begin{code}
primes        ::  [Integer]
primes        =   sieve [2..]

sieve         ::  [Integer] -> [Integer]
sieve (p:xs)  =   p : sieve [x | x <- xs, x `mod` p > 0]
\end{code}
\begin{figure} [ht]
  \centering
  \subcaptionbox{Early state}
    {\includegraphics[scale=.3]{images/sieve-2.pdf}}
  \subcaptionbox{Later state}
    {\includegraphics[scale=.3]{images/sieve-9.pdf}}
  \caption{Sieve of Eratosthenes, see Appendix~\ref{ap:sieve} for a larger example}\label{fig:sieve}
\end{figure}
What happens in the sieve of Eratosthenes in Figure~\ref{fig:sieve} is that
once a number is visited by the top thunk, it is declared a prime, added to the
final list of primes and a thunk is created for it. This thunk traverses down
the infinite list of numbers and filters out all multiples of itself.  Likewise
the top thunk wanders down the resulting list, spawning new thunks for all
numbers it encounters. Because the numbers have been filtered by the thunks spawned before they must be primes.

\subsection{Unboxed Values}
Some basic Haskell data types are defined as follows in GHC:
\begin{code}
data Int      =  I#  Int#

data Integer  =  S#  Int#
              |  J#  Int# ByteArray#

data Word     =  W#  Word#

data Float    =  F#  Float#

data Double   =  D#  Double#
\end{code}
We have already seen \emph{S\#}, which is used to define small integers and
\emph{D\#} for double precision floating point numbers.

The difference between \emph{Int} and \emph{Int\#} is that the first is a boxed
value, the second is unboxed. We can easily visualize what that means:
\begin{Verbatim}
ghci> data Point = Point Int Int
ghci> :view Point 2 3
ghci> data PointU = PointU Int# Int#
ghci> :view PointU 2# 3#
\end{Verbatim}
\begin{figure} [ht]
  \centering
  \includegraphics[scale=.4]{images/boxing.pdf}
  \caption{Boxed and unboxed values}
  \label{fig:boxing}
\end{figure}
In Figure~\ref{fig:boxing} we see that boxed values are not actually stored in
the data constructor, but instead are seperate closures which the data
constructor points to. Unboxed values are stored inside of the data constructor
directly. This makes them interesting for performant code as the overhead of an
indirection and the space for another closure are saved.

\subsection{Double Linked List}

A nice example of the power of lazy evaluation is using it to create a double
linked list\cite{tyingtheknot}:
\begin{code}
data DList alpha = DLNode (DList alpha) alpha (DList alpha) deriving Show

mkDList     ::  [alpha] -> DList alpha
mkDList []  =   error "must have at least one member"
mkDList xs  =   let  (first,last) = go last xs first
                in   first

  where  go  ::  DList alpha -> [alpha] -> DList alpha -> (DList alpha, DList alpha)
         go prev []      next  =   (next,prev)
         go prev (x:xs)  next  =   let  this         = DLNode prev x rest
                                        (rest,last)  = go this xs next
                                   in   (this,last)
\end{code}
The real work is done in the |go| function: A |DLNode| is created using |prev|,
which is the previous list member, and |rest|, which is only defined in the
next line and is the result of calling |go| for the next node. Thanks to lazy
evaluation we can supply |this| to |go| and at the same time define |this|
using the result of the exact same call to |go|.

Connecting the last and the first members of the double linked list also makes
heavy use of lazy evaluation: In |mkDList| |last| is defined as the second
tuple element of a call to |go|, which takes |last| as a parameter. This
technique is called \emph{tying the knot}.

\begin{figure} [ht]
  \centering
  \subcaptionbox{Linear view}
    {\includegraphics[width=\textwidth]{images/dll-linear.pdf}}

  \subcaptionbox{Graph view}
    {\includegraphics[scale=.4]{images/dll-graph.pdf}}
  \caption{Double linked list}\label{fig:dll}
\end{figure}

Trying to print the created double linked list using the \emph{show} function
does not work, because it is cyclic and \emph{show} does not know when to stop.
The structure of the final list can be seen using ghc-vis, as is shown in
Figure~\ref{fig:dll}.

\subsection{IntMap}
Sharing is used in many Haskell libraries to achieve high performance. Consider
for example \emph{IntMap}s:
\begin{Verbatim}
ghci> let x = Data.IntMap.fromList $ zip [1..5] [1..]
ghci> x
fromList [(1,1),(2,2),(3,3),(4,4),(5,5)]
ghci> let y = Data.IntMap.fromList $ zip [6..10] [6..]
ghci> y
fromList [(6,6),(7,7),(8,8),(9,9),(10,10)]
ghci> :view x
ghci> :view y
ghci> :view Data.IntMap.union x y
\end{Verbatim}
\begin{figure} [ht]
  \centering
  \includegraphics[width=\textwidth]{images/intmap.pdf}
  \caption{Sharing in \emph{IntMap}s}\label{fig:intmap}
\end{figure}
The resulting graph is shown in Figure~\ref{fig:intmap}. Only some internal
nodes get newly created for the union of \emph{x} and \emph{y}, most of the
original data structures are reused.

\subsection{Nexuses and Dynamic Programming}
This and the next subsection contain examples from \cite{trouble}. In
\emph{dynamic programming} a memo table is usually used to avoid recomputing
solutions to subproblems. In Haskell one can use another approach to achieve
the same: \emph{Nexus trees}, which are trees with shared nodes.

We define a data type \emph{Tree} for binary trees:
\savecolumns
\begin{code}
data Tree a  =   Empty
             |   Node { left :: Tree a, info :: a, right :: Tree a }
       deriving (Eq, Show)

leaf            ::  forall alpha<.>alpha -> Tree alpha
leaf x          =   Node Empty x Empty
\end{code}
The definition of insertion into a binary tree:
\restorecolumns
\begin{code}
insert          ::  forall alpha<.>Ord alpha => alpha -> Tree alpha -> Tree alpha
insert x Empty  =   leaf x
insert x (Node l k r)
  | x <= k      =   Node (insert x l) k r
  | otherwise   =   Node l k (insert x r)
\end{code}
Now we can create a full binary tree of depth \emph{n} containing the value
\emph{x} using the function \emph{full}:
\restorecolumns
\begin{code}
full            ::  forall alpha<.>Integer -> alpha -> Tree alpha
full 0 x        =   leaf x
full n x        =   Node t x t
  where t = full (n-1) x
\end{code}
\begin{figure} [ht]
  \centering
  \includegraphics[scale=.4]{images/full-shared.pdf}
  \caption{Visualization of \emph{full}: Nodes are shared}\label{fig:full-shared}
\end{figure}
Figure~\ref{fig:full-shared} shows how sharing is used in the evaluation of
|full 4 2.0|: The child node |t| is created once and then shared as the left
and right child, which makes the resulting tree a nexus. Because of this a
full binary tree can be created in $O(\log n)$ time and stored in $O(\log n)$
space, where $n$ is the number of nodes.

Once we change a seemingly slight detail of the definition of \emph{full}, as
we did in \emph{full'} by typing out the identical left and right child of the
created node, the sharing disappears:
\restorecolumns
\begin{code}
full'           ::  forall alpha<.>Integer -> alpha -> Tree alpha
full' 0 x       =   leaf x
full' n x       =   Node (full' (n-1) x) x (full' (n-1) x)
\end{code}
\begin{figure} [ht]
  \centering
  \includegraphics[width=\textwidth]{images/full-unshared.pdf}
  \caption{Visualization of \emph{full'}: Nodes are \emph{not} shared}\label{fig:full-unshared}
\end{figure}
Because of referential transparency this refactoring is allowed and does not
change the semantics of the function. Figure~\ref{fig:full-unshared} reveals
the effect the missing sharing has: We're not getting a nexus anymore. Space
and time used are now linear, even though we are clearly expressing the same
graph, as |full 4 2.0 == full' 4 2.0| evaluates to |True|.

Nexuses provide an alternative way to the list based one for calculating
Fibonacci numbers we visualized in Subsection~\ref{ss:infinite}:
\begin{code}
memofib    ::  Integer -> Tree Integer
memofib 0  =   leaf 0
memofib 1  =   Node (leaf 0) 1 Empty
memofib n  =   node t (left t)
  where t = memofib (n-1)

node       ::  Tree Integer -> Tree Integer -> Tree Integer
node l r   =   Node l (info l + info r) r
\end{code}
\begin{figure} [ht]
  \centering
  \includegraphics[scale=.3]{images/memofib.pdf}
  \caption{Fibonacci numbers using a nexus}\label{fig:memofib}
\end{figure}

The n-th Fibonacci number can be obtained using |info (memofib 8)|. In
Figure~\ref{fig:memofib} the sharing used in \emph{memofib} can be seen.

\subsection{Optimal Bracketing}

Let us consider the problem of \emph{optimal bracketing}: We want to bracket an
expression of the form $x_1 \oplus x_2 \oplus \dotsc \oplus x_n$, where
$\oplus$ is an associative operator. That means the result will be the same, no
matter how we bracket the expression. But computational costs can differ, for
example in a chain matrix multiplication.
\begin{code}
data Expr alpha  =  Const alpha
                 |  Expr alpha :+: Expr alpha
  deriving Show
\end{code}
We can solve the problem using the recursive function |opt|:
\begin{code}
opt            ::  [alpha] -> Expr alpha
opt [x]        =   Const x
opt xs         =   best [opt s1 :+: opt s2 | (s1,s2) <- uncat xs]

uncat          ::  forall alpha<.>[alpha] -> [([alpha],[alpha])]
uncat [x1,x2]  =   [([x1],[x2])]
uncat (x:xs)   =   ([x],xs):map(\(l,r) -> (x:l,r)) (uncat xs)
\end{code}
In this example we consider the expression with the lowest bracketing depth the
best one, as could be desirable for parallel execution:
\begin{code}
best     ::  [Expr alpha] -> Expr alpha
best xs  =   snd $ go xs
  where   go [y]           =  (size y, y)
          go (y:ys)
            | sizeY < sys  =  (sizeY, y)
            | otherwise    =  gys
            where gys@(sys,_) = go ys
                  sizeY = size y

size               ::  Expr a -> Integer
size ((l :+: r))   =   1 + max (size l) (size r)
size (Const _)     =   1
\end{code}
The possible bracketings of $Const 1 \oplus Const 2 \oplus Const 3 \oplus Const 4$ are:
\begin{enumerate}
  \item $((Const 1 \oplus Const 2) \oplus Const 3) \oplus Const 4$
  \item $(Const 1 \oplus Const 2) \oplus (Const 3 \oplus Const 4)$
  \item $Const 1 (\oplus Const 2 \oplus (Const 3 \oplus Const 4))$
\end{enumerate}

The second bracketing is the optimal one by our definiton of |best|, as is
verified by |opt [1..4]| evaluating to |(Const 1 :+: Const 2) :+: (Const 3 :+:
Const 4)|.

But this function takes exponentional time, which is why we are now going to
consider a solution using a nexus:
\savecolumns
\begin{code}
leaf                 ::  alpha -> Tree (Expr alpha)
leaf x               =   Node Empty (Const x) Empty
\end{code}
|leaf x| creates a Leaf node storing the value |x|.
\restorecolumns
\begin{code}
lspine, rspine       ::  forall alpha<.>Tree alpha -> [alpha]
lspine (Empty)       =   []
lspine (Node l x r)  =   lspine l ++ [x]

rspine (Empty)       =   []
rspine (Node l x r)  =   [x] ++ rspine r
\end{code}
The functions |lspine| and |rspine| traverse the left and right side of the
tree respectively, returning the nodes visited.
\restorecolumns
\begin{code}
combine              ::  [(Expr alpha, Expr alpha)] -> Expr alpha
combine              =   best . map (uncurry (:+:))

node                 ::  Tree (Expr alpha) -> Tree (Expr alpha) -> Tree (Expr alpha)
node l r             =   Node l (combine (zip (lspine l) (rspine r))) r
\end{code}
The function |node| constructs a new node for our nexus.
\restorecolumns
\begin{code}
step                 ::  [Tree (Expr alpha)] -> [Tree (Expr alpha)]
step [t]             =   []
step (t1:t2:ts)      =   node t1 t2 : step (t2:ts)
\end{code}
|step| creates the next layer of nodes in our nexus. Sharing is introduced in
the last line by |t2| appearing twice.
\restorecolumns
\begin{code}
build                ::  [Tree (Expr alpha)] -> Tree (Expr alpha)
build [t]            =   t
build ts             =   build (step ts)

bottomup             ::  [alpha] -> Tree (Expr alpha)
bottomup             =   build . map leaf
\end{code}
Finally |bottomup| builds the entire nexus using the funtions defined above.

\begin{figure} [ht]
  \centering
  \includegraphics[scale=.3]{images/bottomup.pdf}
  \caption{Solving the \emph{optimal bracketing} problem using a nexus}\label{fig:bottomup}
\end{figure}

When we now evaluate |bottomup [1..4]| the data structure seen in
Figure~\ref{fig:bottomup} gets created. The resulting tree uses extensive
sharing to construct the optimal solution of the problem from optimal solutions
to subproblems. The solution is obtained by |info (bottomup [1..4])|.

Further examples of consciously using sharing in the form of nexuses are
available in \cite{nexuses}.

% let x = mkTree split "abcd"
% let x = mkTree isegs "abcd"

%\subsection{Sudoku Solver}
%TODO: Add "A simple Sudoku solver" for a good example of lazy evaluation.
%
%%\cite[Chapter~19]{pearls}
%
%\cite{sudoku}

\subsection{Negative Effects of Lazy Evaluation and Sharing}

Unfortunately the effects of lazy evaluation and sharing are not always
beneficial. Consider the follwing example:
\begin{Verbatim}
ghci> let x = let n = [1..5] in (last n, head n)
ghci> :view x
\end{Verbatim}
\begin{figure} [ht]
  \centering
  \begin{subfigure}[b]{.3\linewidth}
    \centering
    \includegraphics[scale=.4]{images/spaceleak-1.pdf}
    \caption{Tuple elements are unevaluated}
  \end{subfigure}
  \begin{subfigure}[b]{.3\linewidth}
    \centering
    \includegraphics[scale=.4]{images/spaceleak-2.pdf}
    \caption{First tuple element was evaluated (|fst x|)}
  \end{subfigure}
  \begin{subfigure}[b]{.3\linewidth}
    \centering
    \includegraphics[scale=.4]{images/spaceleak-3.pdf}
    \caption{Second tuple element was evaluated (|snd x|)}
  \end{subfigure}
  \caption{Space leak}\label{fig:spaceleak}
\end{figure}
\begin{figure} [ht]
  \centering
  \begin{subfigure}[b]{.3\linewidth}
    \centering
    \includegraphics[scale=.4]{images/nospaceleak-1.pdf}
    \caption{Tuple elements are unevaluated}
  \end{subfigure}
  \begin{subfigure}[b]{.3\linewidth}
    \centering
    \includegraphics[scale=.4]{images/nospaceleak-2.pdf}
    \caption{First tuple element was evaluated (|fst x|)}
  \end{subfigure}
  \begin{subfigure}[b]{.3\linewidth}
    \centering
    \includegraphics[scale=.4]{images/nospaceleak-3.pdf}
    \caption{Second tuple element was evaluated (|snd x|)}
  \end{subfigure}
  \caption{No more space leak by avoiding sharing}\label{fig:nospaceleak}
\end{figure}
As we can see in Figure~\ref{fig:spaceleak} when the first element of the tuple
|x| gets evaluated the whole list |n| gets evaluated to determine its last
member and thereby stored on the heap. As |n| is shared between the tuple
elements, |head n| now refences the entire list |n| and forces the garbage
collector to leave it on the heap. This would not be necessary, as |head n| can
be obtained from the first list member of |n|, whereas the rest of the list is
not needed anymore. A \emph{space leak} has occured. While the space leak in
this example is negligibly small, much larger ones can occur in practice.  When
we \emph{avoid sharing} this problem does not occur in our example:
\begin{Verbatim}
ghci> let y = (last [1..5], head [1..5])
ghci> :vis y
\end{Verbatim}
The resulting data structure can be seen in Figure~\ref{fig:nospaceleak}. This
would not work anymore when we get passed a list instead. A more general
approach that \emph{explicitly un-shares} data structures is provided by the
experimental |dup|\cite{ghc-dup} operation.
\clearpage % HACK
Another cause for space leaks is lazy evaluation:
\begin{code}
sum    ::  (Num a) => [a] -> a
sum l  =   sum' l 0
 where
   sum'  []      a = a
   sum'  (x:xs)  a = sum' xs (a+x)
\end{code}
This is the definition of |sum| from Haskell's base library. When we evaluate
|sum [1..10^7]| in GHCi the memory usage quickly grows to hundreds of
Megabytes.

What is happening? We can use ghc-vis in combination with GHCi's debugger to
find out:\footnote{Note that the debugger of GHCi can only use breakpoints and
single-stepping in interpreted code, so we have to load our own file containing
the code to be debugged instead of using the |sum| function from the base
library.}
\begin{Verbatim}
ghci> :l sum
[1 of 1] Compiling Main             ( sum.hs, interpreted )
Ok, modules loaded: Main.
ghci> :vis
ghci> :break 6
ghci> sum [1..10^7]
Stopped at sum.hs:6:20-32
_result :: Integer = _
a :: Integer = _
x :: Integer = 1
xs :: [Integer] = _
ghci> :view a
ghci> :clear
ghci> :continue
Stopped at sum.hs:6:20-32
_result :: Integer = _
a :: Integer = _
x :: Integer = 2
xs :: [Integer] = _
ghci> :view a
ghci> :clear
ghci> :continue
Stopped at sum.hs:6:20-32
_result :: Integer = _
a :: Integer = _
x :: Integer = 3
xs :: [Integer] = _
ghci> :view a
\end{Verbatim}
\begin{figure} [ht]
  \subcaptionbox{First stop}
    {\includegraphics[width=.325\textwidth]{images/sum-1.pdf}}
  \subcaptionbox{Second stop}
    {\includegraphics[width=.325\textwidth]{images/sum-2.pdf}}
  \subcaptionbox{Third stop}
    {\includegraphics[width=.325\textwidth]{images/sum-3.pdf}}
  \caption{Space leak caused by lazy evaluation in |sum|}\label{fig:sum}
\end{figure}
The status of |a| at every recursion step can be seen in Figure~\ref{fig:sum}.
The path of unevaluated closures to calculate |a| keeps growing. In the last
line of the definition of |sum'| |(a+x)| is not evaluated and instead the
unevaluated thunk gets passed to the next call to |sum'|. This builds up a big
amount of thunks, even though it could be reduced to a single value.

To fix this we apply the last argument in the call to |sum'| strictly using the
$\$!$ operator:
\begin{code}
sum2    ::  (Num a) => [a] -> a
sum2 l  =   sum' l 0
 where
   sum'  []      a = a
   sum'  (x:xs)  a = sum' xs $! (a+x)
\end{code}
\begin{figure} [ht]
  \begin{subfigure}[b]{.32\textwidth}
    \centering
    \includegraphics[scale=.4]{images/sum2-1.pdf}
    \caption{First stop}
  \end{subfigure}
  \begin{subfigure}[b]{.32\textwidth}
    \centering
    \includegraphics[scale=.4]{images/sum2-2.pdf}
    \caption{Second stop}
  \end{subfigure}
  \begin{subfigure}[b]{.32\textwidth}
    \centering
    \includegraphics[scale=.4]{images/sum2-3.pdf}
    \caption{Third stop}
  \end{subfigure}
  \caption{Avoiding space leak by strict evaluation in |sum2|}\label{fig:sum2}
\end{figure}
This \emph{strict evaluation} causes |a+x| to be evaluated to weak-head normal
form, which means only the actual value gets passed. Old values can then be
removed by the garbage collector. Memory usage is constant with |sum2|, and the
correct behavior can be seen in Figure~\ref{fig:sum2}, which has been recorded
in the same way as we just did for |sum|.

GHC uses optimization techniques which can automatically prevent space leaks
like this. When optimization is enabled during compilation the space leak
disappears.

\section{Combination with GHCi's Debugger}

Evaluating a value to weak-head normal form works very well when visualizing
the evaluation of lists or other composite data types. But for elementary data
types like Integers even that may be too much and a smaller evaluation may be
more appropriate. ghc-vis can be combined with GHCi's debugger using the
supplied \texttt{:su}, \texttt{:asu} and \texttt{:tsu} commands, which allow
single-stepping through the evaluation of expressions and watching the data
structures transform during this.

%ghc-vis can be combined with the debugger of GHCi to visualize data structures
%while debugging, as we already saw in the last example.

We are going to use a file called \emph{test.hs} in this example to illustrate
this:
\begin{code}
f         ::  [Integer] -> [Integer]
f (x:xs)  =   (2 * x) : f xs
f []      =   []
\end{code}
\begin{Verbatim}
ghci> :l test.hs
ghci> :vis
ghci> let x = [1..3]
ghci> let y = f x
ghci> let z = sum y
ghci> :view x
ghci> :view y
ghci> :view z
ghci> :su z
\end{Verbatim}
\begin{figure} [ht]
  \hfill
  \subcaptionbox{Early state}
    {\includegraphics[scale=.4]{images/l5.pdf}}
  \hfill
  \subcaptionbox{Later state}
    {\includegraphics[scale=.23]{images/l11.pdf}}
  \hfill
  \caption{Combination with GHCi's debugger, see Appendix~\ref{ap:debugger} for a larger example}\label{fig:debugger}
\end{figure}
Using \texttt{:su} we can now single-step through the evaluation of |z| and
watch the data structures changing during it. Two intermediate steps can be
seen in Figure~\ref{fig:debugger}.

The available commands for combining ghc-vis with GHCi's debugger are described
in Table~\ref{tab:debugger}.

\begin{table} [ht]
  \centering
  \begin{tabular}{lp{12.3cm}}
    \toprule
    Command & Description \\
    \midrule
    \texttt{:su x}  & Same as \texttt{:step x}: Single-step through
                      evaluation of \emph{x}, but the visualization gets
                      updated automatically \\
    \texttt{:cu}    & Same as \texttt{:continue}: Continue to next breakpoint,
                      but the visualization gets updated automatically \\
    \texttt{:asu x} & Single-step through evaluation of \emph{x} on any key
                      pressed, quit with \texttt{q} \\
    \texttt{:tsu t x} & Single-step through evaluation of \emph{x} with one
                        step every \emph{t} seconds until a key is pressed \\
    \bottomrule
  \end{tabular}
  \caption{Debugging commands of ghc-vis}
  \label{tab:debugger}
\end{table}

\section{Usage as a Library}
Although ghc-vis is mainly meant to be used in GHCi it can also be used as a
library in regular Haskell programs which are then run or compiled by GHC to
inspect expressions inside of them at runtime\footnote{Currently ghc-vis only
works correctly in compiled programs when they have been compiled with the
\emph{-threaded} flag}:

\begin{minipage}{0.29\textwidth}
\begin{code}
import GHC.Vis

main = do
  let a  =  "teeest"
  let b  =  [1..3]
  let c  =  b ++ b
  let d  =  [1..]
  putStrLn $ show $ d !! 1

  visualization
  view  a  "a"
  view  b  "b"
  view  c  "c"
  view  d  "d"

  getChar
  switch

  getChar
\end{code}
\end{minipage}
\begin{figure} [ht]
  \centering
  \includegraphics[scale=.4]{images/library.pdf}
  \caption{ghc-vis as a library}\label{fig:library}
\end{figure}

The output on the screen looks exactly like the one we are used from inside of
GHCi, but some optimizations stand out, which are not performed in GHCi: Notice
for example how letters are shared within |a| in Figure~\ref{fig:library}.

The GHCi commands of ghc-vis closely resemble the library interface, as is
shown in Table~\ref{tab:library}.

\begin{table} [ht]
  \centering
  \begin{tabular}{lll}
    \toprule
    Method call & Type & Corresponding GHCi command \\
    \midrule
    |visualization| & |IO ()| & \texttt{:vis} \\
    |view x "x"| & |a -> String -> IO ()| & \texttt{:view x} \\
    |eval "t0"| & |String -> IO ()| & \texttt{:eval t0} \\
    |switch| & |IO ()| & \texttt{:switch} \\
    |update| & |IO ()| & \texttt{:update} \\
    |clear| & |IO ()| & \texttt{:clear} \\
    |export "file.pdf"| & |String -> IO ()| & \texttt{:export file.pdf} \\
    \bottomrule
  \end{tabular}
  \caption{Public library interface}
  \label{tab:library}
\end{table}

\newpage
\section{Source Code}
\label{ch:Evaluation:sec:section2}

\begin{table} [ht]
  \centering
  \begin{tabular}{llrr}
    \toprule
    Project & File & Code & Comments \\
    \midrule
    ghc-vis & Vis.hs & 180 & 70 \\
    & Vis/Internal.hs & 393 & 184 \\
    & View/List.hs & 276 & 129 \\
    & View/Graph.hs & 117 & 53 \\
    & View/Graph/Parser.hs & 79 & 49 \\
    & View/Common.hs & 43 & 27 \\
    & Types.hs & 70 & 29 \\
    & ghci & 11 & 9 \\
    \cmidrule(l){3-4}
    & & 1169 & 550 \\
    \midrule
    xdot & Parser.hs & 213 & 46 \\
    & Viewer.hs & 156 & 56 \\
    & Types.hs & 35 & 13 \\
    & Demo.hs & 96 & 36 \\
    \cmidrule(l){3-4}
    & & 500 & 151 \\
    \bottomrule
  \end{tabular}
  \caption{Source code statistics}
  \label{tab:source}
\end{table}

Statistics of the source code for ghc-vis and xdot are presented in
Table~\ref{tab:source}. They were created using GHC's \emph{count\_lines}.

\newpage
\section{Performance}
\label{ch:Evaluation:sec:section3}

ghc-vis is not written with big data structures in mind, as their
visualizations would become two crowded to understand easily. The measurements
of ghc-vis' performance in Table~\ref{tab:performance} indicate that the
HeapMap data structure is not restricting the performance, but instead the
complex drawing operations take most of the time.

\begin{table} [ht]
  \centering
  \begin{tabular}{lrr}
    \toprule
    Operation & Evaluated members & Runtime [ms]\\
    \midrule
    walkHeap & 0 & 0.02 \\
    & 25 & 0.13 \\
    & 50 & 0.36 \\
    & 75 & 0.70 \\
    & 100 & 1.19 \\
    \midrule
    xDotParse & 0 & 13.46 \\
    & 25 & 22.82 \\
    & 50 & 36.52 \\
    & 75 & 53.11 \\
    & 100 & 69.70 \\
    \midrule
    linear view & 0 & 6.68 \\
    & 25 & 9.51 \\
    & 50 & 11.68 \\
    & 75 & 19.71 \\
    & 100 & 22.04 \\
    \midrule
    graph view & 0 & 36.87 \\
    & 25 & 125.68 \\
    & 50 & 239.17 \\
    & 75 & 353.40 \\
    & 100 & 472.54 \\
    \midrule
    \bottomrule
  \end{tabular}
  \caption{Runtime of various operations on a partially evaluated list}
  \label{tab:performance}
\end{table}

These measurements were taken using \emph{Criterion}\cite{criterion} on a
machine with an Intel~Core2 processor clocked at 2.5~GHz, running Linux~3.5.3
and GHC~7.4.2 using the x86\hbox{-}64 instruction set.
